/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.utils;

import java.util.Deque;
import java.util.LinkedList;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;

/**
 * Utility class managing SWT graphics context state by storing and restoring state attributes
 * of the underlying graphics context using the provided public methods.
 * <P>
 * Note that it is the obligation of the user of this class to ensure that methods are called in
 * order of the stack principle. Otherwise, {@link ClassCastException}s will occur.
 * 
 * @author hoelzl
 */
public final class GCStateManager {
	/** The graphics context to be managed. */
	private final GC gc;
	/** The stack used to store the current gc state attributes. */
	private final Deque<Object> state = new LinkedList<Object>();

	/** Constructor. */
	public GCStateManager(GC gc) {
		this.gc = gc;
	}

	/** Returns the graphics context. */
	public GC GC() {
		return gc;
	}

	/** Stores the alpha value. */
	public GCStateManager alphaStoreAndSet(int alpha) {
		state.push(new Integer(gc.getAlpha()));
		gc.setAlpha(alpha);
		return this;
	}

	/** Restores the alpha value. */
	public GCStateManager alphaRestore() {
		gc.setAlpha((Integer)state.pop());
		return this;
	}

	/** Stores the current foreground color and creates and activates the given color. */
	public GCStateManager foregroundStoreAndCreate(RGB rgb) {
		state.push(gc.getForeground());
		gc.setForeground(new Color(Display.getCurrent(), rgb));
		return this;
	}

	/** Restores the previous foreground color and disposes the current one. */
	public GCStateManager foregroundRestoreAndDispose() {
		gc.getForeground().dispose();
		gc.setForeground((Color)state.pop());
		return this;
	}

	/** Stores the current background color and creates and activates the given color. */
	public GCStateManager backgroundStoreAndCreate(RGB rgb) {
		state.push(gc.getBackground());
		gc.setBackground(new Color(Display.getCurrent(), rgb));
		return this;
	}

	/** Restores the previous background color and disposes the current one. */
	public GCStateManager backgroundRestoreAndDispose() {
		gc.getBackground().dispose();
		gc.setBackground((Color)state.pop());
		return this;
	}

	/** Stores the current font and creates and activates the given font. */
	public GCStateManager fontStoreAndCreate(FontData fontData) {
		state.push(gc.getFont());
		gc.setFont(new Font(Display.getCurrent(), fontData));
		return this;
	}

	/** Restores the previous font and disposes the current one. */
	public GCStateManager fontRestoreAndDispose() {
		gc.getFont().dispose();
		gc.setFont((Font)state.pop());
		return this;
	}

	/** Stores the line width and style and activates the given ones. */
	public GCStateManager lineStoreAndSet(int width, int style) {
		state.push(gc.getLineWidth());
		gc.setLineWidth(width);
		state.push(gc.getLineStyle());
		gc.setLineStyle(style);
		return this;
	}

	/** Restores the line width and style. */
	public GCStateManager lineRestore() {
		gc.setLineStyle((Integer)state.pop());
		gc.setLineWidth((Integer)state.pop());
		return this;
	}
}
